"use strict";
/**
 * @file This plugin is to access classification image data from different sources. Make sure that
 * the data is conform to expectation.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const pipcook_core_1 = require("@pipcook/pipcook-core");
const glob_promise_1 = __importDefault(require("glob-promise"));
const path = __importStar(require("path"));
const fs = __importStar(require("fs-extra"));
class DataLoader extends pipcook_core_1.ImageDataLoader {
    constructor(dataPairs) {
        super();
        pipcook_core_1.shuffle(dataPairs);
        this.dataPairs = dataPairs;
    }
    len() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.dataPairs.length;
        });
    }
    getItem(id) {
        return __awaiter(this, void 0, void 0, function* () {
            return {
                data: this.dataPairs[id].image,
                label: this.dataPairs[id].label
            };
        });
    }
    setItem(id, sample) {
        return __awaiter(this, void 0, void 0, function* () {
            this.dataPairs[id].image = sample.data;
            this.dataPairs[id].label = sample.label;
        });
    }
}
/**
 * merge all possible values of labels. Get the map between label and numeric value
 * @param dataPath
 */
const getLabelMap = (dataPath) => __awaiter(void 0, void 0, void 0, function* () {
    const labelSet = new Set();
    const trainFileNames = yield glob_promise_1.default(path.join(dataPath, 'train', '*.xml'));
    for (let j = 0; j < trainFileNames.length; j++) {
        const fileName = trainFileNames[j];
        const imageData = yield pipcook_core_1.parseAnnotation(fileName);
        imageData.annotation.object.forEach((object) => {
            labelSet.add(object.name[0]);
        });
    }
    const labelArray = Array.from(labelSet);
    const labelMap = {};
    labelArray.forEach((label, index) => {
        labelMap[label] = index;
    });
    return labelMap;
});
const getValidPair = (dataPath, labelMap) => __awaiter(void 0, void 0, void 0, function* () {
    const cocoJson = {
        info: {
            "description": "dataset generated by pipcook",
            "url": "http:\/\/mscoco.org",
            "version": "1.0", "year": 2014,
            "contributor": "Microsoft COCO group",
            "date_created": "2015-01-27 09:11:52.357475"
        },
        images: [],
        licenses: [
            {
                "url": "http:\/\/creativecommons.org\/licenses\/by-nc-sa\/2.0\/",
                "id": 1,
                "name": "Attribution-NonCommercial-ShareAlike License"
            }
        ],
        annotations: [],
        categories: []
    };
    const annotationPaths = yield glob_promise_1.default(path.join(dataPath, '*.xml'));
    const pairs = [];
    for (let i = 0; i < annotationPaths.length; i++) {
        const fileName = annotationPaths[i];
        const imageData = yield pipcook_core_1.parseAnnotation(fileName);
        const imageName = imageData.annotation.filename[0];
        const imageItem = {
            license: 1,
            file_name: imageName,
            coco_url: 'default',
            id: i + 1
        };
        if (imageData.annotation.size && imageData.annotation.size[0]) {
            imageItem.width = parseInt(imageData.annotation.size[0].width[0]);
            imageItem.height = parseInt(imageData.annotation.size[0].height[0]);
        }
        if (fs.existsSync(path.join(dataPath, imageName))) {
            cocoJson.images.push(imageItem);
            imageData.annotation.object.forEach((object) => {
                const label = {
                    name: object.name[0],
                    categoryId: labelMap[object.name[0]]
                };
                const category = cocoJson.categories.find((e) => e.name === object.name[0]);
                let id;
                if (category) {
                    id = category.id;
                }
                else {
                    id = labelMap[object.name[0]];
                    cocoJson.categories.push({
                        id: labelMap[object.name[0]],
                        name: object.name[0],
                        supercategory: object.name[0]
                    });
                }
                const cocoItem = {
                    id: cocoJson.annotations.length + 1,
                    image_id: i + 1,
                    category_id: id,
                    iscrowd: Number((object.iscrowd && object.iscrowd[0])) || 0
                };
                if (object.segmentation && object.segmentation[0]) {
                    if (object.segmentation[0].counts) {
                        cocoItem.segmentation = {
                            counts: object.segmentation[0].counts[0],
                            size: [
                                Number(object.segmentation[0].size[0]),
                                Number(object.segmentation[0].size[1])
                            ]
                        };
                    }
                    else if (object.segmentation[0].polygon) {
                        cocoItem.segmentation = object.segmentation[0].polygon[0];
                    }
                }
                if (object.bndbox && object.bndbox[0]) {
                    const width = parseInt(object.bndbox[0].xmax[0]) - parseInt(object.bndbox[0].xmin[0]);
                    const height = parseInt(object.bndbox[0].ymax[0]) - parseInt(object.bndbox[0].ymin[0]);
                    cocoItem.bbox = [parseInt(object.bndbox[0].xmin[0]), parseInt(object.bndbox[0].ymin[0]), width, height];
                    cocoItem.area = Number(width * height);
                }
                cocoJson.annotations.push(cocoItem);
                if (object.bndbox) {
                    label.bndbox = {
                        xmin: Number(object.bndbox[0].xmin[0]),
                        ymin: Number(object.bndbox[0].ymin[0]),
                        xmax: Number(object.bndbox[0].xmax[0]),
                        ymax: Number(object.bndbox[0].ymax[0])
                    };
                }
                if (object.segmentation) {
                    label.segmentation = object.segmentation[0];
                }
                pairs.push({
                    annotation: fileName,
                    image: path.join(dataPath, imageName),
                    label
                });
            });
        }
    }
    yield fs.outputJSON(path.join(dataPath, 'annotation.json'), cocoJson);
    return pairs;
});
/**
 * The plugin used to access data from different sources. It will detect all possible values of labels and
 * merge them into numeric expressions.
 */
const cocoDataAccess = (args) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    const { dataDir } = args;
    const labelMap = yield getLabelMap(dataDir);
    const labelArray = Object.keys(labelMap).sort((k1, k2) => labelMap[k1] - labelMap[k2]);
    const trainPair = yield getValidPair(path.join(dataDir, 'train'), labelMap);
    const validationPair = yield getValidPair(path.join(dataDir, 'validation'), labelMap);
    const testPair = yield getValidPair(path.join(dataDir, 'test'), labelMap);
    const isBitMask = (_a = trainPair[0].label.segmentation) === null || _a === void 0 ? void 0 : _a.hasOwnProperty('counts');
    const trainLoader = new DataLoader(trainPair);
    const validationLoader = new DataLoader(validationPair);
    const testLoader = new DataLoader(testPair);
    const result = {
        metadata: {
            labelMap,
            isBitMask,
            labelArray
        },
        dataStatistics: [],
        validationResult: {
            result: true
        },
        trainAnnotationPath: path.join(dataDir, 'train', 'annotation.json'),
        validationAnnotationPath: path.join(dataDir, 'validation', 'annotation.json'),
        testAnnotationPath: path.join(dataDir, 'test', 'annotation.json')
    };
    if (trainPair.length > 0) {
        result.trainLoader = trainLoader;
    }
    if (validationPair.length > 0) {
        result.validationLoader = validationLoader;
    }
    if (testPair.length > 0) {
        result.testLoader = testLoader;
    }
    return result;
});
exports.default = cocoDataAccess;
//# sourceMappingURL=index.js.map